from _Framework.ControlSurface import ControlSurface
from _Framework.InputControlElement import *
from MrDeviceComponent import MrDeviceComponent
from _Framework.ChannelTranslationSelector import ChannelTranslationSelector
from _Framework.EncoderElement import EncoderElement
from _Framework.ButtonElement import ButtonElement
from MrSysex import MrSysex
from MrMixerComponent import MrMixerComponent
from const import *
import Live

class MrLemur(ControlSurface):

	def __init__(self, c_instance):
		ControlSurface.__init__(self, c_instance)
		self._setupMrLemur()

	def _setupMrLemur(self):
		self.set_suppress_rebuild_requests(True)
		self._suppress_session_highlight = True
		self._suppress_send_midi = True
		MrSysex.set_midi_callback(self._send_midi)
		MrSysex.set_log(self.log_message)
		self._mixer = MrMixerComponent(NUM_TRACKS)
		self.setUpDeviceControl()
		#Add transport controls
		for component in self.components:
			component.set_enabled(False)
		self.set_suppress_rebuild_requests(False)
		self.refresh_state()
		self._mixer._send_init()
		self.show_message("Let's Go!")

	def setUpDeviceControl(self):
		device_param_controls = []
		for index in range(8):
			device_param_controls.append(EncoderElement(1,0,index,Live.MidiMap.MapMode.absolute))
		device = MrDeviceComponent()
		device.set_parameter_controls(tuple(device_param_controls))
		device.set_bank_nav_buttons(ButtonElement(False,1,0,LEFT_NAV),ButtonElement(False,1,0,RIGHT_NAV))
		device.set_on_off_button(ButtonElement(False,1,0,DEVICE_ON_OFF))
		device.set_lock_button(ButtonElement(False,1,0,DEVICE_LOCK))
		device_translation_selector = ChannelTranslationSelector()
		device_translation_selector.set_controls_to_translate(tuple(device_param_controls))
		self.set_device_component(device)

	def refresh_state(self):
		ControlSurface.refresh_state(self)
		self.schedule_message(5,self._init)

	def _init(self):
		self._suppress_send_midi = False
		self._suppress_session_highlight = True
		for component in self.components:
			component.set_enabled(True)
			
	def suggest_input_port(self):
	    return 'Daemon Input '
	    
	def suggest_output_port(self):
	    return 'Daemon Output '
	    
	def handle_sysex(self,sysex):
	    if(sysex[1] == MIXER_SYSEX):
	        self._mixer.handle_sysex(sysex[1:])
	
	def _do_send_midi(self,midi_event_bytes):
	    if not self._suppress_send_midi:
	        self._c_instance.send_midi(midi_event_bytes)
	        return True

	def disconnect(self):
		ControlSurface.disconnect(self)
		MrSysex.release_attributes()
		self._mixer.disconnect(self._mixer)
